<?php

/**
 * Payment Modul für girosolution.de Directdebit.
 *
 * @version   4.0.0
 * @author    Thorsten Marx <thorsten.marx@girosolution.de>
 * @copyright 2011-2015 GiroSolution AG
 * @link      http://www.girosolution.de
 * @link      http://www.xtc-modified.org
 * @license   http://opensource.org/licenses/gpl-2.0.php GNU Public License
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */
// Lib laden.
if (file_exists(DIR_FS_CATALOG . 'includes/classes/giroconnect/class.giroconnect.php')) {
  include_once DIR_FS_CATALOG . 'includes/classes/giroconnect/class.giroconnect.php';
} else {
  include_once '../includes/classes/giroconnect/class.giroconnect.php';
}

if (file_exists(DIR_FS_CATALOG . 'includes/classes/giroconnect/library/GiroCheckout_SDK/GiroCheckout_SDK.php') === true) {
  include_once DIR_FS_CATALOG . 'includes/classes/giroconnect/library/GiroCheckout_SDK/GiroCheckout_SDK.php';
} else {
  include_once '../includes/classes/giroconnect/library/GiroCheckout_SDK/GiroCheckout_SDK.php';
}

/**
 * Class to manage a directdebit payment method.
 *
 * Extends the giroconnect_main
 *
 * @author GiroSolution AG
 * @package GiroCheckout
 * @copyright Copyright (c) 2014, GiroSolution AG
 */
class gc_directdebit extends giroconnect_main {

  /**
   * Constructor method.
   * Set the payment type and payment method credentials.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return void
   */
  function gc_directdebit() {
    global $order;

    $this->_setCode("gc_directdebit");
    $this->_setLang();

    $this->title = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_TEXT_TITLE');
    $this->description = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_TEXT_DESCRIPTION');
    $this->description_checkout = xtc_image('includes/classes/giroconnect/library/logos/Logo_EC_50_px.jpg', '', '', '', 'style="display: block;"') . giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_TEXT_DESCRIPTION');

    if (defined('MODULE_PAYMENT_' . strtoupper($this->code) . '_STATUS')) {
      $this->sort_order = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_SORT_ORDER');
      $this->enabled = ((giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_STATUS') == 'True') ? true : false);
      $this->redirect = true;
      $this->tmpOrders = true;
      $this->form_action_url = true;
      $this->tmpStatus = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_ORDER_STATUS_ID_TEMP');
    }

    if (defined('_VALID_XTC')) {
      if ($this->allowed != '') {
        $this->title .= ' (' . $this->allowed . ')';
      }
    }

    if (is_object($order)) {
      $this->update_status();
    }
  }

  //end gc_directdebit()

  /**
   * Set array with the title, description and fields to display.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return array
   */
  function selection() {
    $pic = xtc_image('includes/classes/giroconnect/library/logos/Logo_EC_50_px.jpg', '', '', '', 'style="display: block;"');
    $desc = $pic . $this->description;

    $databankcheck = isset($_SESSION['giroconnect']['directdebit_useBankAccount']) ? $_SESSION['giroconnect']['directdebit_useBankAccount'] : '';
    $accountHolder = isset($_SESSION['giroconnect']['directdebit_accountholder']) ? $_SESSION['giroconnect']['directdebit_accountholder'] : '';
    if (empty($databankcheck)) {
      $iban = isset($_SESSION['giroconnect']['directdebit_iban']) ? $_SESSION['giroconnect']['directdebit_iban'] : '';
      $bankCode = '';
      $accountNumber = '';
      $strVisDivIban = "block";
      $strVisDivBankDatas = "none";
      $bCheckboxChecked = false;
    } else {
      $bankCode = isset($_SESSION['giroconnect']['directdebit_bankcode']) ? $_SESSION['giroconnect']['directdebit_bankcode'] : '';
      $accountNumber = isset($_SESSION['giroconnect']['directdebit_bankaccount']) ? $_SESSION['giroconnect']['directdebit_bankaccount'] : '';
      $iban = '';
      $strVisDivIban = "none";
      $strVisDivBankDatas = "block";
      $bCheckboxChecked = true;
    }

    $output = '<br/>';
    $output.='<script>'
            . 'function changeDatasDirectDebit(){'
            . 'var a = document.getElementById("gc_bankaccount_checkbox");'
            . 'if(a.checked == true){'
            . 'document.getElementById("divIbanDirectdebit").style.display = "none";'
            . 'document.getElementById("divBankDatasDirectdebit").style.display = "block";'
            . '}else{'
            . 'document.getElementById("divIbanDirectdebit").style.display = "block";'
            . 'document.getElementById("divBankDatasDirectdebit").style.display = "none";'
            . '}'
            . '}'
            . '</script>';


    return array(
        'id' => $this->code,
        'module' => $this->title,
        'description' => $desc . $output,
        'fields' => array(
            array(
                'title' => '',
                'field' => xtc_draw_checkbox_field('gc_bankaccount_checkbox', 'cbBankDatasDirectdebit', $bCheckboxChecked, 'id="gc_bankaccount_checkbox" onclick="changeDatasDirectDebit()"') . giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_BANKACCOUNTBANKCODE_DESC')
            ),
            array(
                'title' => '',
                'field' => '<p>' . giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_ACCOUNTHOLDER_DESC') . xtc_draw_input_field("{$this->code}_accountholder", $accountHolder, 'size=34 maxlength="27"') . '</p>'
            ),
            array(
                'title' => '',
                'field' => '<div id ="divIbanDirectdebit" style="display: ' . $strVisDivIban . ';"> <p>IBAN: ' . xtc_draw_input_field("{$this->code}_iban", $iban, 'size="34"') . '</p></div>'
            ),
            array(
                'title' => '',
                'field' => '<div id ="divBankDatasDirectdebit" style="display: ' . $strVisDivBankDatas . ';"><p>' . giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_BANKACCOUNT_DESC') . xtc_draw_input_field("{$this->code}_bankaccount", $accountNumber, 'maxlength="10"') . '</p><p>&nbsp;</p><p>' . giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_BANKCODE_DESC') . xtc_draw_input_field("{$this->code}_bankcode", $bankCode, 'maxlength="8"') . '</p></div>'
            ),
        )
    );
  }

  //end selection()

  /**
   * Valid the payment datas inserted by the user.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return boolean
   */
  function pre_confirmation_check() {
    global $_POST;

    $bankCode = $_POST["{$this->code}_bankcode"];
    $accountNumber = $_POST["{$this->code}_bankaccount"];
    $iban = $_POST["{$this->code}_iban"];
    $databankcheck = $_POST["gc_bankaccount_checkbox"];
    $accountHolder = $_POST["{$this->code}_accountholder"];

    $_SESSION['giroconnect']['directdebit_bankcode'] = $bankCode;
    $_SESSION['giroconnect']['directdebit_bankaccount'] = $accountNumber;
    $_SESSION['giroconnect']['directdebit_iban'] = $iban;
    $_SESSION['giroconnect']['directdebit_useBankAccount'] = $databankcheck;
    $_SESSION['giroconnect']['directdebit_accountholder'] = $accountHolder;


    //Checks if all input datas are correct.
    if (!empty($accountHolder)) {
      if (empty($databankcheck)) {
        if (!empty($iban)) {
          return true;
        } else {
          $strResponseMsg = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_IBAN_MISSING');
          xtc_redirect(str_replace('&amp;', '&', xtc_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&error=' . urlencode($strResponseMsg), 'SSL')));
          return false;
        }
      } else {
        if (!empty($bankCode) && !empty($accountNumber)) {
          return true;
        } else {
          $strResponseMsg = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_BANKACCOUNTBANKCODE_MISSING');
          xtc_redirect(str_replace('&amp;', '&', xtc_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&error=' . urlencode($strResponseMsg), 'SSL')));
          return false;
        }
      }
    } else {
      $strResponseMsg = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_ACCOUNTHOLDER_MISSING');
      xtc_redirect(str_replace('&amp;', '&', xtc_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $this->code . '&error=' . urlencode($strResponseMsg), 'SSL')));
      return false;
    }
  }

  //end pre_confirmation_check()

  /**
   * Functions to do when the button is pressed.
   * Set hidden fields with payment datas.
   * Get the total.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return void
   */
  function process_button() {
    global $order, $_POST, $xtPrice;

    if ($_SESSION['customers_status']['customers_status_show_price_tax'] == 0 && $_SESSION['customers_status']['customers_status_add_tax_ot'] == 1) {
      $total = $order->info['total'] + $order->info['tax'];
    } else {
      $total = $order->info['total'];
    }

    //sores total because of bug due to wrong pricing after an aborted transaction
    $_SESSION['giroconnect']['total'] = round($xtPrice->xtcCalculateCurrEx($total, $order->info['currency']), $xtPrice->get_decimal_places($order->info['currency']));


    $_SESSION['giroconnect']['directdebit_bankcode'] = $_POST["{$this->code}_bankcode"];
    $_SESSION['giroconnect']['directdebit_bankaccount'] = $_POST["{$this->code}_bankaccount"];
    $_SESSION['giroconnect']['directdebit_iban'] = $_POST["{$this->code}_iban"];
    $_SESSION['giroconnect']['directdebit_useBankAccount'] = $_POST["gc_bankaccount_checkbox"];

    $process_button_string = xtc_draw_hidden_field("{$this->code}_bankcode", $_POST["{$this->code}_bankcode"]) .
            xtc_draw_hidden_field("{$this->code}_bankaccount", $_POST["{$this->code}_bankaccount"]) .
            xtc_draw_hidden_field("{$this->code}_iban", $_POST["{$this->code}_iban"]) .
            xtc_draw_hidden_field("{$this->code}_accountholder", $_POST["{$this->code}_accountholder"]) .
            xtc_draw_hidden_field("gc_bankaccount_checkbox", $_POST["gc_bankaccount_checkbox"]);

    return $process_button_string . '<script src="https://bankauswahl.giropay.de/widget/v1/jquery-1.10.2.min.js"></script><script type="text/javascript">
$(document).ready(function()
    {
$("#checkout_confirmation").find("input[type=image]").click(function(){

        document.getElementById("checkout_confirmation").submit();
	$(this).prop( "disabled", true );

});
});
</script>';
  }

  //end process_button()

  /**
   * Initialize the payment
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return boolean
   */
  function payment_action() {
    global $_POST, $order, $insert_id;

    $parts = explode('-', $_SESSION[$this->_gc_cartID]);
    $order_id = (int) end($parts);

    // Fix for XTC Bug
    //
    // $order->info['total'] is in 'after_process' String with currency ->
    // 1.02 EUR
    // so it has to be set here
    $amount = $_SESSION['giroconnect']['total'];
    $currency = $_SESSION['currency'];
    $bankCode = $_POST["{$this->code}_bankcode"];
    $accountNumber = $_POST["{$this->code}_bankaccount"];
    $iban = $_POST["{$this->code}_iban"];
    $accountHolder = $_POST["{$this->code}_accountholder"];
    $databankcheck = $_POST["gc_bankaccount_checkbox"];
    $transaction_id = $order_id;
    $vwz = utf8_encode(trim(substr('Best. ' . $order_id.' ' . STORE_NAME, 0, 27)));
    $merchantId = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_MERCHANTID');
    $projectId = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_PROJECTID');
    $password = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($this->code) . '_PASSWORD');
    $sourceId = $this->getGcSource();

    if (strlen($accountHolder > 27)) {
      $accountHolder = substr($accountHolder, 0, 27);
    }

    $pm = $this->code;
    $paymentSuccess = false;
    $paymentMsg = '';
    $insert_id = $transaction_id;
    $delete_order = ((giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($pm) . '_DELETE_ORDER') == 'True') ? true : false);


    //Insert new giroconnect transaction with init status.
    //xtc_db_query('delete from gc_transactions where order_id = "' . xtc_db_input((int) $transaction_id) . '"');
    $sql_data_array = array(
        'order_id' => $transaction_id,
        'rc' => 'null',
        'rc_msg' => 'null',
        'init_start' => strtotime("now"),
        'init_end' => 'null',
        'time_finished' => 'null',
        'reference' => 'null',
        'backend_tx_id' => 'null',
        'result_payment' => 'null',
        'tx_status' => 1,
        'payment_code' => $this->code,
        'comment' => $vwz,
    );

    xtc_db_perform('gc_transactions', $sql_data_array);
    $gc_tx_id = xtc_db_insert_id();

    try {

      //Send request to Girocheckout based on which radiobutton is checked ($databankcheck).
      if (empty($databankcheck)) {
        $request = new GiroCheckout_SDK_Request('directDebitTransaction');
        $request->setSecret($password);
        $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $gc_tx_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('iban', $iban)
                ->addParam('accountHolder', $accountHolder)
                ->addParam('sourceId', $sourceId)
                ->submit();
      } else {
        $request = new GiroCheckout_SDK_Request('directDebitTransaction');
        $request->setSecret($password);
        $request->addParam('merchantId', $merchantId)
                ->addParam('projectId', $projectId)
                ->addParam('merchantTxId', $gc_tx_id)
                ->addParam('amount', $amount * 100)
                ->addParam('currency', $currency)
                ->addParam('purpose', $vwz)
                ->addParam('bankcode', $bankCode)
                ->addParam('bankaccount', $accountNumber)
                ->addParam('accountHolder', $accountHolder)
                ->addParam('sourceId', $sourceId)
                ->submit();
      }

      if ($request->requestHasSucceeded() && $request->paymentSuccessful()) {
        $iReturnCode = $request->getResponseParam('rc');
        $paymentMsg = $request->getResponseMessage($iReturnCode, strtoupper($this->language));
        $paymentSuccess = true;
        $strReference = $request->getResponseParam('reference');
        $strBackendTxId = $request->getResponseParam('backendTxId');
        $newStatus = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($pm) . '_ORDER_STATUS_ID_OK');
        $comment_order_status_history = 'payment with ' . $pm . ' successful';

        //Change giroconnect transaction to successful status.
        xtc_db_query("update gc_transactions set time_finished = " . strtotime("now") . ", init_end = " . strtotime("now") . ""
                . ", result_payment = " . xtc_db_input($request->getResponseParam('resultPayment'))
                . ", result_payment_msg = '" . xtc_db_input($request->getResponseMessage($request->getResponseParam('resultPayment'), strtoupper($this->language)))
                . "', reference = '" . xtc_db_input($strReference)
                . "', backend_tx_id = '" . xtc_db_input($strBackendTxId)
                . "', rc = " . xtc_db_input($request->getResponseParam('rc'))
                . ", rc_msg = '" . xtc_db_input($request->getResponseMessage($request->getResponseParam('rc'), strtoupper($this->language)))
                . "', tx_status = 2 where tx_id = '" . xtc_db_input($gc_tx_id) . "'");
      } else {
        if (!$request->requestHasSucceeded()) {
          $iReturnCode = $request->getResponseParam('rc');
          $paymentMsg = $request->getResponseMessage($iReturnCode, strtoupper($this->language));
          $iResultPayment = 'null';
        } elseif (!$request->paymentSuccessful()) {
          $iResultPayment = $request->getResponseParam('resultPayment');
          $paymentMsg = $request->getResponseMessage($iResultPayment, strtoupper($this->language));
          $iReturnCode = '0';
        }

        $newStatus = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($pm) . '_ORDER_STATUS_ID_ERROR');
        $strReference = $request->getResponseParam('reference');
        $strBackendTxId = $request->getResponseParam('backendTxId');
        $comment_order_status_history = $pm . ' payment error:  ' . $paymentMsg;

        //Change giroconnect transaction to failed status.
        xtc_db_query("update gc_transactions set time_finished = " . strtotime("now") . ", init_end = " . strtotime("now") . ""
                . ", result_payment = " . xtc_db_input($iResultPayment)
                . ", result_payment_msg = '" . xtc_db_input($request->getResponseMessage($request->getResponseParam('resultPayment'), strtoupper($this->language)))
                . "', reference = '" . xtc_db_input($strReference)
                . "', backend_tx_id = '" . xtc_db_input($strBackendTxId)
                . "', rc = " . xtc_db_input($iReturnCode)
                . ", rc_msg = '" . xtc_db_input($request->getResponseMessage($request->getResponseParam('rc'), strtoupper($this->language)))
                . "', tx_status = 3 where tx_id = '" . xtc_db_input($gc_tx_id) . "'");
      }
    } catch (Exception $e) {
      $newStatus = giroconnect_main::constant('MODULE_PAYMENT_' . strtoupper($pm) . '_ORDER_STATUS_ID_ERROR');
      $paymentMsg = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, strtoupper($this->language));
      $comment_order_status_history = $pm . ' payment error:  ' . $paymentMsg;

      //Change giroconnect transaction to failed status.
      xtc_db_query("update gc_transactions set time_finished = " . strtotime("now") . ", init_end = " . strtotime("now") . ""
              . ", result_payment = 5100"
              . ", result_payment_msg = '" . xtc_db_input($paymentMsg)
              . "', rc = 5100"
              . ", rc_msg = '" . xtc_db_input($paymentMsg)
              . "', tx_status = 3 where tx_id = '" . xtc_db_input($gc_tx_id) . "'");
    }

    //Notification to the database for Directdebit
    // Konfiguration laden.
    include_once 'includes/application_top.php';

    // Bestellung nicht gefunden, Benachrichtigung soll nicht mehr erfolgen.
// Eintrag zur Historie der Bestellung hinzuf�gen.
    $sql_data_array = array(
        'orders_id' => (int) $transaction_id,
        'orders_status_id' => $newStatus,
        'date_added' => 'now()',
        'customer_notified' => (int) $paymentSuccess,
        'comments' => $comment_order_status_history
    );
    xtc_db_perform(TABLE_ORDERS_STATUS_HISTORY, $sql_data_array);


    // Bestell-Status in Tabelle orders updaten.
    xtc_db_query("update " . TABLE_ORDERS . " set orders_status = '" . xtc_db_input($newStatus) . "', last_modified = now() where orders_id = '" . xtc_db_input((int) $transaction_id) . "'");


    if ($paymentSuccess) {

      // send email
      $smarty = new Smarty;
      require_once DIR_WS_CLASSES . 'order.php';
      $insert_id = $transaction_id;

      include_once 'send_order.php';

      /*       * ************************************************************ */
    } else if ($delete_order) {
      $check_query = xtc_db_query('select orders_id from ' . TABLE_ORDERS . ' where orders_id = "' . xtc_db_input((int) $transaction_id) . '" limit 1');
      if (xtc_db_num_rows($check_query) > 0) {
        xtc_db_query('delete from ' . TABLE_ORDERS . ' where orders_id = "' . xtc_db_input((int) $transaction_id) . '"');
        xtc_db_query('delete from ' . TABLE_ORDERS_TOTAL . ' where orders_id = "' . xtc_db_input((int) $transaction_id) . '"');
        xtc_db_query('delete from ' . TABLE_ORDERS_STATUS_HISTORY . ' where orders_id = "' . xtc_db_input((int) $transaction_id) . '"');
        xtc_db_query('delete from ' . TABLE_ORDERS_PRODUCTS . ' where orders_id = "' . xtc_db_input((int) $transaction_id) . '"');
        xtc_db_query('delete from ' . TABLE_ORDERS_PRODUCTS_ATTRIBUTES . ' where orders_id = "' . xtc_db_input((int) $transaction_id) . '"');
        xtc_db_query('delete from ' . TABLE_ORDERS_PRODUCTS_DOWNLOAD . ' where orders_id = "' . xtc_db_input((int) $transaction_id) . '"');
      }
      unset($_SESSION['cart_GC_ID']);
    }

    //Redirection to payment method selection page or success page for Directdebit
    // Parameter lesen.
    if (!$paymentSuccess) {
      $strUrl = str_replace('&amp;', '&', xtc_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error=' . $pm . '&error=' . urlencode($paymentMsg), 'SSL'));
    } else {
      $_SESSION['cart']->reset(true);
      // unregister session variables used during checkout
      unset($_SESSION['sendto']);
      unset($_SESSION['billto']);
      unset($_SESSION['shipping']);
      unset($_SESSION['payment']);
      unset($_SESSION['comments']);
      unset($_SESSION['last_order']);

      unset($_SESSION['cart_GC_ID']);
      unset($_SESSION['giroconnect']);

      //GV Code Start
      if (isset($_SESSION['credit_covers'])) {
        unset($_SESSION['credit_covers']);
      }

      $strUrl = xtc_href_link(FILENAME_CHECKOUT_SUCCESS, "order_success_text=" . urlencode($paymentMsg), 'SSL');
    }
    xtc_redirect($strUrl);

    return true;
  }

  //end payment_action()
}
